#include <stdafx.h>
#include <stdio.h>

#include "lmcons.h"
#include "lmmsg.h"

#define XP_NOERROR              0
#define XP_ERROR                1
#define MAXCOLNAME				25
#define MAXNAME					25
#define MAXTEXT					255

#define MAX_SERVER_ERROR 20000
#define XP_SAMPLE_ERROR MAX_SERVER_ERROR+1

// These defintions were included in the SQL 7.0 version of srv.h but not in VS version
#define SS_MAJOR_VERSION    8
#define SS_MINOR_VERSION    00
#define SS_LEVEL_VERSION    0000
#define SS_MINIMUM_VERSION  "7.00.00.0000"
#define ODS_VERSION			((SS_MAJOR_VERSION << 24)  | (SS_MINOR_VERSION << 16))


#ifdef __cplusplus
extern "C" {
#endif

RETCODE __declspec(dllexport) uxp_NetMessage(SRV_PROC *srvproc);
ULONG __declspec(dllexport) __GetXpVersion();
void printUsage (SRV_PROC* pSrvProc);
void printError (SRV_PROC *pSrvProc, CHAR* szErrorMsg);

#ifdef __cplusplus
}
#endif


ULONG __declspec(dllexport) __GetXpVersion()
{
    return ODS_VERSION;
}


RETCODE __declspec(dllexport) uxp_NetMessage(SRV_PROC *srvproc)
{
  // Count up the number of input parameters
  if (srv_rpcparams(srvproc) != 3)
  {
	// Send error message and return
	printUsage (srvproc);
	return (XP_ERROR);
  }

  // Allocate memory for the string parameters
  LPWSTR awcFromName = 
	(LPWSTR)LocalAlloc(LPTR,((srv_paramlen(srvproc,1)+1) 
	 * sizeof(WCHAR)));
  LPWSTR awcToName = 
	(LPWSTR)LocalAlloc(LPTR,((srv_paramlen(srvproc,2)+1) 
	* sizeof(WCHAR)));
  LPWSTR awcMesgBuffer = 
	(LPWSTR)LocalAlloc(LPTR,((srv_paramlen(srvproc,3)+1) 
	* sizeof(WCHAR)));

  // Convert the parameters to UNICODE
  MultiByteToWideChar(   
	CP_ACP,										// code page
	0,											// character-type
	(const char *)srv_paramdata(srvproc,1),		// address of char
	srv_paramlen(srvproc,1)+1,					// number of bytes
	awcFromName,								// address of wchar
	(srv_paramlen(srvproc,1)+1)*sizeof(WCHAR));	// size of buffer
			
	
  MultiByteToWideChar(
	CP_ACP,										// code page
	0,											// character-type
	(const char *)srv_paramdata(srvproc,2),		// address of char
	srv_paramlen(srvproc,2)+1,					// number of bytes
	awcToName,									// address of wchar
	(srv_paramlen(srvproc,2)+1)*sizeof(WCHAR));	// size of buffer
			
	
  MultiByteToWideChar(
	CP_ACP,										// code page
	0,											// character-type
	(const char *)srv_paramdata(srvproc,3),		// address of char
	srv_paramlen(srvproc,3)+1,					// number of bytes
	awcMesgBuffer,								// address of wchar
	(srv_paramlen(srvproc,3)+1)*sizeof(WCHAR)); // size 

	NET_API_STATUS nasStatus;

// Send a message to the target system
  nasStatus = NetMessageBufferSend(NULL,
				awcToName,
				awcFromName,
				(LPBYTE)awcMesgBuffer,
				srv_paramlen(srvproc,3)*sizeof(WCHAR));

 // Free the memory use by the parameters
  LocalFree(awcFromName);
  LocalFree(awcToName);
  LocalFree(awcMesgBuffer);

  // Return the status information
  if (nasStatus == 0)
	return (XP_NOERROR);
  else
  {
	printError (srvproc, "NetMessageBufferSend error");
	return (XP_ERROR);
  }
}

// send XP usage info to client
void printUsage (SRV_PROC *pSrvProc)
{
// usage: 
// exec uxp_NetMessage <@param1 input, @param2 input, @param3 input>

  srv_sendmsg(pSrvProc, SRV_MSG_ERROR, XP_SAMPLE_ERROR, SRV_INFO, 1,
    NULL, 0, (DBUSMALLINT) __LINE__, 
	(char *)printf ("Usage: exec uxp_NetMessge ",
	"@param1 input, @param2 input, @param3 input>"),
    SRV_NULLTERM);
  srv_senddone(pSrvProc, (SRV_DONE_ERROR | SRV_DONE_MORE), 0, 0);
}


// send szErrorMsg to client
void printError (SRV_PROC *pSrvProc, CHAR* szErrorMsg)
{
  srv_sendmsg(pSrvProc, SRV_MSG_ERROR, XP_SAMPLE_ERROR, SRV_INFO, 1,
    NULL, 0, (DBUSMALLINT) __LINE__, 
    szErrorMsg,
    SRV_NULLTERM);

    srv_senddone(pSrvProc, (SRV_DONE_ERROR | SRV_DONE_MORE), 0, 0);
}